"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveOrg = exports.resolveAppNameForEnvironment = exports.fetchAppForProject = exports.fetchSfdxProject = exports.fetchOrgId = exports.fetchOrg = void 0;
const core_1 = require("@salesforce/core");
const heroku_variant_1 = require("./heroku-variant");
async function fetchOrg(aliasOrUsername) {
    // if `aliasOrUsername` is null here, Org.create will pull the default org from the surrounding environment
    return core_1.Org.create({
        aliasOrUsername,
    });
}
exports.fetchOrg = fetchOrg;
async function fetchOrgId(aliasOrUsername) {
    const org = await fetchOrg(aliasOrUsername);
    return org.getOrgId();
}
exports.fetchOrgId = fetchOrgId;
async function fetchSfdxProject() {
    const project = await core_1.SfdxProject.resolve();
    return project.resolveProjectConfig();
}
exports.fetchSfdxProject = fetchSfdxProject;
async function fetchAppForProject(client, projectName, orgAliasOrUsername) {
    const orgId = await fetchOrgId(orgAliasOrUsername);
    const { data } = await client.get(`/sales-org-connections/${orgId}/apps/${projectName}`, {
        headers: Object.assign({}, heroku_variant_1.default('evergreen')),
    });
    return data;
}
exports.fetchAppForProject = fetchAppForProject;
async function resolveAppNameForEnvironment(appNameOrAlias) {
    // Check if the environment provided is an alias or not, to determine what app name we use to attempt deletion
    const info = await core_1.GlobalInfo.getInstance();
    const matchingAlias = info.aliases.getValue(appNameOrAlias);
    let appName;
    if (matchingAlias) {
        appName = matchingAlias;
    }
    else {
        appName = appNameOrAlias;
    }
    return appName;
}
exports.resolveAppNameForEnvironment = resolveAppNameForEnvironment;
async function resolveOrg(orgId) {
    // We perform this check because `Org.create` blows up with a non-descriptive error message if you
    // just assume the target org is set
    const config = await core_1.ConfigAggregator.create();
    const targetOrg = config.getPropertyValue('target-org');
    if (!orgId && !targetOrg) {
        throw new Error('Attempted to resolve an org without an org ID or target-org value');
    }
    const infos = await core_1.AuthInfo.listAllAuthorizations();
    if (infos.length === 0)
        throw new Error('No connected orgs found');
    if (orgId) {
        const matchingOrg = infos.find((info) => info.orgId === orgId);
        if (matchingOrg) {
            return await core_1.Org.create({ aliasOrUsername: matchingOrg.username });
        }
    }
    return core_1.Org.create({ aliasOrUsername: targetOrg });
}
exports.resolveOrg = resolveOrg;
